/*

USAGE -- Reads one or more Maximus or Binkley logs and generates a usage graph

Version 1.7  (8/25/92)

Written by Bob Quinlan of Austin, Texas, USA
Sysop of Red October at 512-834-2593 (1:382/111)

Copyright 1992 by Bob Quinlan

Compatible with Maximus 2.00 and 2.01 and BinkleyTerm 2.55 and 2.56


This program reads one or more Maximus or Binkley logs and generates a
BBS-format usage graph based on the data.  By default it will read
MAX.LOG in the current directory and write USAGE.BBS.  The following
switches allow you to configure it for your system and preferences:

    /Cx=y       Set color number 'x' to Avatar color 'y'.  'x' values
                mean the following:
                    0 = default color
                    1 = title
                    2 = frame
                    3 = reference text (%)
                    4 = reference lines (%)
                    5 = hour text
                    6 = hour dots (odd hour values)
                    7 = hour lines
                    8 = data bars
                    9 = enter prompt
                Avatar color codes are given in two-digit hexadecimal.
                The first digit sets the background color (0-7 only) and
                the second digit sets the foreground color (0-F).  Each
                digit corresponds to the following colors:
                    0 = black
                    1 = blue
                    2 = green
                    3 = cyan
                    4 = red
                    5 = magenta
                    6 = brown
                    7 = white
                    8 = grey
                    9 = bright blue
                    A = bright green
                    B = bright cyan
                    C = bright red
                    D = bright magenta
                    E = yellow
                    F = bright white
                Example: To set the data bars to yellow on blue you
                would use the switch /C8=1E.

    /Lfile      Log file to process.  Note that the /L parameter can be
                used more than once to specify several logs.  This
                feature is particularly useful if you have multiple
                lines with separate logs.

    /Ttitle     Title for graph.  You may want to supply multiple word
                values for title.  You can separate the words by spaces
                or by underscores (which will be replaced by spaces).
                For example, both of these lines would produce the same
                results:
                    usage /lmax.log /tRed October Usage
                    usage /lmax.log /tRed_October_Usage

    /Ufile      The file to which the usage graph will be written. The
                .BBS extension is not automatically added.

    /V          Verbose diagnostic mode.  Prints out each log entry that
                triggers a starting or ending time.  Prints the elapsed
                time between each start/end pair.  Not recommended for
                normal use.

If you use WFC within Maximus feed USAGE the Maximus log.  If you use
Binkley to answer the phone feed it the Binkley log.  You can mix log
types if you have lines with different configurations.

As an example, my system has two lines, one running Binkley and the
other running Maximus with WFC.  I start a new log every month and keep
the previous month's around as well.  Here is how I generate my overall
usage graph:

usage /lbink01l.old /lbink01.log /lmax02l.old /lmax02.log
/tRed October Overall Usage

(That's all one line in my batch file.)

I have the Log Mode in Maximus set to Verbose and the LogLevel in
Binkley set to 5.  If you use other settings USAGE may not register all
of your system's activity.


USAGE returns ERRORLEVEL 0 after a successful run.  ERRORLEVEL 1 is
returned to indicate an error.

NOTICE:  You may use, copy, and distribute this program freely as long
as you insure that both the executable and the documentation (.DOC)
files are included in the distribution package.  The source code does
not need to be included.  You may modify this program and document, so
long as reasonable credit is given to the original author if a
substantial portion of the original remains intact.  The author is not
responsible for any losses which may occur either directly or indirectly
as a result of using this program.

HISTORY:
Version 1.7   (8/25/92) -- Added the ability to pass multiple-word
                           parameters using spaces.
Version 1.6   (6/03/92) -- Adjusted graph to start at midnight.
Version 1.5   (5/29/92) -- Improved Binkley support.  Added verbose
                           diagnostic mode.
Version 1.4   (5/27/92) -- Allow custom color selections.
Version 1.3   (5/26/92) -- Removed event and keyboard switches.
                           Added support for Binkley logs.
                           Added log file sharing (again).
Version 1.2   (5/14/92) -- Removed log file sharing due to compiler bug.
Version 1.1   (5/13/92) -- Added log file sharing.
Version 1.0   (4/22/92) -- Original release.  Written in Borland C.

Large memory model
*/


#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <io.h>
#include <share.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <time.h>


#define MAXLINE     (512)
#define MAXPATH     (128)
#define MAXTITLE    (72)
#define MAXDATE     (7)
#define MAXLOGS     (12)
#define NUM_COLORS  (10)

#define DAYMINS     (1440)
#define COLS        (72)
#define CDIV        (DAYMINS/COLS)


main(int argc, char *argv[])
{
char   logfile[MAXLOGS][MAXPATH];
FILE   *logfile_fp;
char   logs = 0;
int    curlog = 0;

char   usefile[MAXPATH] = {"USAGE.BBS"};
FILE   *usefile_fp;

char   comment[MAXLINE];

char   title[MAXTITLE] = {""};

int    online;

int    calls = 0;
int    days = 0;
int    time = 0;

int    verbose = 0;

char   colors[NUM_COLORS] =
            {
            0x07,  /*  0: default color  */
            0x0f,  /*  1: title  */
            0x09,  /*  2: frame  */
            0x0a,  /*  3: reference text  */
            0x02,  /*  4: reference lines  */
            0x0b,  /*  5: hour text  */
            0x09,  /*  6: hour dots  */
            0x03,  /*  7: hour lines  */
            0x09,  /*  8: data bars  */
            0x02   /*  9: enter prompt  */
            };

int    use[COLS];

char   out[22][78];
char   color[22][78];

char   last[MAXDATE];
char   now[MAXDATE];

int    addtime;
int    start;
int    end;

char   curcolor = 0;

char   param = '\0';

char   line[MAXLINE];
char   *ch;
int    i, j, k;


/***********/
/*  USAGE  */
/***********/

printf("USAGE 1.7 -- Copyright 1992 by Bob Quinlan (8/25/92)\n\n");

/*  Process switches  */
for (i=1; i<argc; i++)
    {
    /*  Convert previous param to uppercase so single-pass switches will
        not recognize it more than once  */
    param = toupper(param);
    /*  If new parameter set it to lower case for first pass  */
    if (argv[i][0] == '/')
        param = tolower(argv[i][1]);
    switch (param)
        {
        case 'c':  /*  color  */
            if (sscanf(argv[i]+2, "%x=%x", &j, &k) >= 2)
                if ((j < NUM_COLORS) && (k <= 0xff))
                    colors[j] = k;
                else
                    fprintf(stderr, "USAGE: Illegal value in %s\n", argv[i]);
            else
                fprintf(stderr, "USAGE: Illegal format in %s\n", argv[i]);
            break;
        case 'l':  /*  log file  */
            if (logs < MAXLOGS)
                strncpy(logfile[logs++], argv[i]+2, MAXPATH);
            else
                fprintf(stderr, "USAGE: %s exceeds log limit\n",
                      argv[i]+2);
            break;
        case 'T':  /*  title: append additional words  */
            strncat(title, " ", MAXTITLE);
            /*  Fall through to next case!  */
        case 't':  /*  title  */
            strncat(title, argv[i]+((islower(param) != 0)*2), MAXTITLE);
            while ((ch = strchr(title, '_')) != NULL)
                *ch = ' ';
            break;
        case 'u':  /*  use file  */
            strncpy(usefile, argv[i]+2, MAXPATH);
            break;
        case 'v':
            verbose = 1;
            break;
        default:
            fprintf(stderr, "USAGE: Unknown switch: %s\n", argv[i]);
            break;
        }
    }

/*  If not defined supply defaults  */
if (*title == '\0')
    strcpy(title, "SYSTEM USAGE");
if (!logs)
    strcpy(logfile[logs++], "MAX.LOG");

/*  Initialize usage table  */
for (i=0; i<COLS; i++)
    use[i] = 0;

/*  Process all log files  */
while ((logfile_fp = _fsopen(logfile[curlog], "rb", SH_DENYNO)) != NULL)
    {
    printf("Processing %s\n", logfile[curlog]);

    online = 0;
    *last = '\0';
    while (fgets(line, MAXLINE, logfile_fp) != NULL)
        {
        /*  Ignore blank lines  */
        if (*line != '\r')
            {
            /*  Keep track of the current date  */
            strncpy(now, line+2, MAXDATE);
            now[MAXDATE-1] = '\0';
            /*  Increase counts when date changes  */
            if (strcmp(now, last) != 0)
                {
                addtime += DAYMINS;
                days++;
                strcpy(last, now);
                printf("    %s\n", now);
                }
            }

        /*  Strip and downcase the comment section of the entry  */
        strcpy(comment, line+23);
        strlwr(comment);

        /*  If already processing usage...  */
        if (online)
            {
            /*  All usage ends with the next log block  */
            if ((strncmp(comment, "begin, ", 7) == 0) ||
                  (strncmp(comment, "connect", 7) == 0) ||
                  (strncmp(comment, "re-enabling system ", 19) == 0) ||
                  (strncmp(comment, "end of wazoo", 12) == 0))
                {
                calls++;
                end = atoi(line+9)*60+atoi(line+12)+addtime;
                time += end-start+1;
                /*  Increment usage once per minute  */
                for (i=start; i<=end; i++)
                    {
                    use[(i%DAYMINS)/CDIV]++;
                    }
                online = 0;
                if (verbose)
                    printf("%s---------------------->elapsed: %d\n", line,
                          end-start+1);
                }
            }
        if (!online)
            {
            /*  If looking for new usage...  */
            if ((strncmp(comment, "connect", 7) == 0) ||
                  (strstr(comment, "with errorlevel") != NULL))
                {
                addtime = 0;
                start = atoi(line+9)*60+atoi(line+12);
                online = 1;
                if (verbose)
                    printf("%s", line);
                }
            }
        }

    /*  Close log file  */
    fclose(logfile_fp);

    /*  Check for last log processed  */
    if (++curlog >= logs)
        break;
    }

/*  Check for finish before last log processed  */
if (curlog < logs)
    {
    fprintf(stderr, "USAGE: Unable to open the log file: %s\n",
          logfile[curlog]);
    exit(1);
    }

/*  Initialize virtual screen  */
for (i=0; i<22; i++)
    for (j=0; j<78; j++)
        {
        out[i][j] = ' ';
        color[i][j] = colors[0];
        }

/*  Draw frame  */
for (i=77-COLS; i<77; i++)
    {
    out[0][i] = 205;
    color[0][i] = colors[2];
    out[20][i] = 205;
    color[20][i] = colors[2];
    }
for (i=1; i<20; i++)
    {
    out[i][76-COLS] = 186;
    color[i][76-COLS] = colors[2];
    out[i][77] = 186;
    color[i][77] = colors[2];
    }

/*  Draw corners  */
out[0][4] = 201;
color[0][4] = colors[2];
out[0][77] = 187;
color[0][77] = colors[2];
out[20][4] = 200;
color[20][4] = colors[2];
out[20][77] = 188;
color[20][77] = colors[2];

/*  Draw 20% references  */
for (i=1; i<5; i++)
    {
    /*  Draw reference text  */
    out[i*4][73-COLS] = ((5-i)*2)+'0';
    color[i*4][73-COLS] = colors[3];
    out[i*4][74-COLS] = '0';
    color[i*4][74-COLS] = colors[3];
    out[i*4][75-COLS] = '%';
    color[i*4][75-COLS] = colors[3];
    /*  Draw reference lines  */
    for (j=77-COLS; j<77; j++)
        {
        out[i*4][j] = 196;
        color[i*4][j] = colors[4];
        }
    }

/*  Draw hour references  */
for (i=0; i<24; i++)
    {
    if (i%2)
        {
        /*  Draw dots for odd hours  */
        out[21][77-COLS+COLS/24*i] = 249;
        color[21][77-COLS+COLS/24*i] = colors[6];
        }
    else
        {
        /*  Draw numbers for even hours  */
        k = ((i+11)%12)+1;
        /*  Check for one or two digits  */
        if (k > 9)
            {
            out[21][77-COLS+COLS/24*i] = (k/10)+'0';
            color[21][77-COLS+COLS/24*i] = colors[5];
            out[21][78-COLS+COLS/24*i] = (k%10)+'0';
            color[21][78-COLS+COLS/24*i] = colors[5];
            }
        else
            {
            out[21][77-COLS+COLS/24*i] = (k%10)+'0';
            color[21][77-COLS+COLS/24*i] = colors[5];
            }
        }
    /*  Draw hour lines  */
    if ((i%4 == 0) && (i > 0))
        for (j=1; j<20; j++)
            {
            out[j][77-COLS+COLS/24*i] = 179;
            color[j][77-COLS+COLS/24*i] = colors[7];
            }
    }

/*  Graph data  */
for (i=0; i<COLS; i++)
    {
    /*  Divide use counts by days (for all logs) and minutes per column  */
    /*  39L = 19 rows * 2 + 1  */
    use[i] = use[i]*39L/(days*CDIV);
    /*  Clip to maximum height (just in case)  */
    if (use[i] > 38)
        use[i] = 38;
    j = 0;
    while (j*2 < use[i])
        {
        if (j*2+1 < use[i])
            out[19-j][77-COLS+i] = 219;  /*  full-height block  */
        else
            out[19-j][77-COLS+i] = 220;  /*  half-height block  */
        color[19-j][77-COLS+i] = colors[8];
        j++;
        }
    }

/*  Open use (output) file  */
if ((usefile_fp = _fsopen(usefile, "w+b", SH_DENYRW)) == NULL)
    {
    fprintf(stderr, "USAGE: Unable to open the use file: %s\n", usefile);
    exit(1);
    }

/*  Write [cls] and [moreoff] tokens  */
fprintf(usefile_fp, "\x0c\x0b");

/*  Write centered title  */
fprintf(usefile_fp, "\x16\x01%c   ", colors[1]);
j = 39-strlen(title)/2;
for (i=0; i<j; i++)
    putc(' ', usefile_fp);
fprintf(usefile_fp, "%s\r\n", title);

/*  Write data  */
for (i=0; i<22; i++)
    {
    for (j=0; j<78; j++)
        {
        /*  Write color change tokens only when necessary  */
        if (color[i][j] != curcolor)
            {
            curcolor = color[i][j];
            fprintf(usefile_fp, "\x16\x01%c", curcolor);
            }
        putc(out[i][j], usefile_fp);
        }
    fprintf(usefile_fp, "\r\n");
    }

/*  Add color and [enter] tokens  */
fprintf(usefile_fp, "\x16\x01%c     \x01", colors[9]);

/*  Close use file  */
fclose(usefile_fp);

return 0;
}


